import csv
import pandas as pd
# Xms and Xmx may need to be increased to run this, and 64 bit python as well

#############################
# Copyright (C) 2020 by Clockworks Signal Processing LLC http://clk.works
# Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted.
# 
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, 
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
# CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
# OF THIS SOFTWARE.
#
# Original Author: Elizabeth LaMacchia
#
# This version release 7-may-2020
# See Clockworks appnote0001 for a descritpion and use of this program.
# settings are all made via variables.
#
# Threshold should be the midpoint of the signal.
# expected frequency should mathc the exact measured frequency or a small bias will creep in
# To get TIE from the output of this program please see the spreadsheet examples.

threshold = .0  # for a scope it may be the scope's raw value, so actual would be 10x this (with 10x probe)
freqexp = 1000 # the expected frequency of the edges. Suggest running a 2nd time with the actual from measurements

# no .ext on filename, we'll add suffixes later

filename= r'30hz.0001'  # you can put a full path here with \ for windows
input_file = filename  + '.csv'

# rise | fall
edge = 'rise'

# don't process header
nNumRowToSkip = 6  # match to your source data

# have to deal with variable number of columns.  Col 1 = time, col 2 = val. Some rows could have 3 or 4 cols.
df = pd.read_csv(input_file, skiprows=[i for i in range(0,nNumRowToSkip)], header=None, names=['Time', 'Value', 'ch2'] )
print("csv read complete")
#df.columns = ['Time', 'Value', 'ch2']
df = df.drop(['ch2'], axis = 1)  #would be better to not waste time reading it on the 1st place but so far can't make that work

df['Time'] = df['Time'].astype('float64')
df['Value'] = df['Value'].astype('float64')

ExpectedTime = 1/freqexp
print("Expected period (usec): " + str(ExpectedTime*1000000) )

df.loc[df['Value'] > threshold, 'CurrentState'] = 'high'
df.loc[df['Value'] <= threshold, 'CurrentState'] = 'low'

df['PrevState'] = df.CurrentState.shift(1)
df['PrevValue'] = df.Value.shift(1)
df['PrevTime'] = df.Time.shift(1)

if edge == 'rise':
    df.loc[(df['CurrentState'] == 'high') & (df['PrevState'] == 'low'), 'CrossPt'] = 'Y'

if edge == 'fall':
    df.loc[(df['CurrentState'] == 'low') & (df['PrevState'] == 'high'), 'CrossPt'] = 'Y'

df = df[df.CrossPt == 'Y']

#linear interpolation to find x: exact time crossing threshold
slope = (df['Value']-df['PrevValue'])/(df['Time']-df['PrevTime'])
exacttime = (threshold - df['PrevValue'])/slope + df['PrevTime']

df.insert(loc=len(df.columns), column='TimeAtThreshold', value=exacttime)

df['DeltaT'] = df.TimeAtThreshold - df.TimeAtThreshold.shift(1)

df = df.drop(['CurrentState', 'PrevState', 'PrevValue', 'PrevTime', 'CrossPt'], axis = 1)

df['SubExpT'] = df.DeltaT - ExpectedTime

NumberOfEdges = str(len(df.index))

print(df)

print("Number of edges is: " + NumberOfEdges)

df.to_csv(filename +'_out.csv')

# that's all
